/* Simple interfaces to various protocols.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 *
 * Based on the original code of Services by Andy Church.
 *
 */
 
#include "services.h"

/* Makes an permanent ban from all the servers. Assumes that the matching clients are killed. */

void s_akill(char *user, char *host, char *who, time_t when, time_t expires, char *reason)
{
#if defined(IRC_BAHAMUT)
	send_cmd(NULL, "AKILL %s %s %d %s %ld :%s", host, user, 86400*2, who, time(NULL), reason);
#elif defined(IRC_UNREAL)
	send_cmd(NULL, "TKL + G %s %s %s %ld %ld :%s", user, host, who, 
		time(NULL)+86400*2, /* Avoids filling the akill list of servers too much */
		when, reason);
#elif defined(IRC_DREAMFORGE)
	send_cmd(NULL, "AKILL %s %s :%s", host, user, reason);
#endif	
}

/*************************************************************************/

/* Sends a channel MODE message */

void s_mode(char const *source, char const *chan, char const *mode, char const *target) 
{
#ifdef IRC_BAHAMUT
	send_cmd(source, "MODE %s 0 %s%s%s", chan, mode, 
			(target ? " " : ""), (target ? target : ""));
#else
	send_cmd(source, "MODE %s %s%s%s", chan, mode, 
			(target ? " " : ""), (target ? target : ""));
#endif
}

/*************************************************************************/

/* Sends a channel MODE message (the mode parameter is prefixed
 * by + automatically) */

void s_mode_plus(char const *source, char const *chan, char mode, char const *target) 
{
#ifdef IRC_BAHAMUT
	send_cmd(source, "MODE %s 0 +%c%s%s", chan, mode, 
			(target ? " " : ""), (target ? target : ""));
#else
	send_cmd(source, "MODE %s +%c%s%s", chan, mode, 
			(target ? " " : ""), (target ? target : ""));
#endif
}

/*************************************************************************/

/* Sends a channel MODE message (the mode parameter is prefixed
 * by - automatically) */

void s_mode_minus(char const *source, char const *chan, char mode, char const *target) 
{
#ifdef IRC_BAHAMUT
	send_cmd(source, "MODE %s 0 -%c%s%s", chan, mode, 
			(target ? " " : ""), (target ? target : ""));
#else
	send_cmd(source, "MODE %s -%c%s%s", chan, mode, 
			(target ? " " : ""), (target ? target : ""));
#endif
}

/*************************************************************************/

/* Removes a permanent ban from all the servers. */

void s_rakill(char *user, char *host)
{
#if defined(IRC_UNREAL)
	send_cmd(NULL, "TKL - G %s %s %s", user, host, s_OperServ);
#else
	send_cmd(NULL, "RAKILL %s %s", host, user);
#endif
}

/*************************************************************************/

void s_sgline(char const *mask, char const *reason)
{
#if defined(IRC_BAHAMUT)
	User *u;

	send_cmd(ServerName, "SGLINE %d :%s:%s", strlen(mask), mask, reason);
	
	/* Do things properly: kill all corresponding users as this is 
	   unfortunately not done by the IRCd :/ */
	for (u = firstuser(); u; u = nextuser()) {
		if (match_wild_nocase(mask, u->realname)) {
			slog("Killing G-Lined user %s.", u->nick);
			send_cmd(NULL, "SVSKILL %s :G-Lined: %s", u->nick, reason);
		}
	}
#elif defined(IRC_UNREAL)
	char *rreason = sstrdup(reason), *end = rreason;

	while (*end) {
		if (*end == ' ')
			*end = '_';
		end++;
	}

	send_cmd(ServerName, "SVSNLINE %s :%s", rreason, mask);
#endif	
}

/*************************************************************************/

void s_sqline(char *mask, char *reason)
{
#ifdef IRC_BAHAMUT
	if (*mask == '#') {
		int i;
		Channel *c, *next;
		
		char *av[3];
		struct c_userlist *cu, *cunext;
		
		send_cmd(NULL, "SQLINE %s :%s", mask, reason);
		
		for (i = 0; i < 1024; i++) {
			for (c = chanlist[i]; c; c = next) {
				next = c->next;
				
				if (!match_wild_nocase(mask, c->name))
					continue;
				
				for (cu = c->users; cu; cu = cunext) {
	    			cunext = cu->next;
	    			
	    			if (is_oper(cu->user))
	    				continue;
	    				
	    			av[0] = c->name;
	    			av[1] = cu->user->nick;
	    			av[2] = reason;
	    			send_cmd(s_OperServ, "KICK %s %s :Q-Lined: %s", av[0], av[1], av[2]);
	    			do_kick(s_ChanServ, 3, av);
				}
			}
		}
	} else {
#endif
		int i;
		User *u, *next;
		
		send_cmd(NULL, "SQLINE %s :%s", mask, reason);

		for (i = 0; i < 1024; i++) {
			for (u = userlist[i]; u; u = next) {
				next = u->next;
				if (match_wild_nocase(mask, u->nick)) {
					slog("Killing Q-Lined user %s.", u->nick);
#ifdef IRC_BAHAMUT
					send_cmd(NULL, "SVSKILL %s :Q-Lined: %s", u->nick, reason);
#else
					kill_user(s_OperServ, u->nick, reason);			
#endif
				}
			}
		}
#ifdef IRC_BAHAMUT
	}
#endif
}

/*************************************************************************/

void s_svsmode_user(char const *source, char const *nick, time_t timestamp, char const *modes, char const *arg)
{
#if defined(IRC_BAHAMUT)
	send_cmd(ServerName, "SVSMODE %s %ld %s%s%s", nick, timestamp, modes, (arg ? " " : ""), (arg ? arg : ""));
#elif defined(IRC_UNREAL)
	send_cmd(source, "SVS2MODE %s %s%s%s", nick, modes, (arg ? " " : ""), (arg ? arg : ""));
#else
	send_cmd(ServerName, "SVSMODE %s %s%s%s", nick, modes, (arg ? " " : ""), (arg ? arg : ""));
#endif
}

/*************************************************************************/

void s_svsnoop(char *server, int set)
{
	send_cmd(NULL, "SVSNOOP %s %s", server, (set ? "+" : "-"));	
}

/*************************************************************************/

void s_szline(char *mask, char *who, time_t when, time_t expires, char *reason)
{	
#if defined(IRC_BAHAMUT)
	s_akill("*", mask, who, when, expires, reason);
#elif defined(IRC_UNREAL)
	send_cmd(NULL, "TKL + Z * %s %s %ld %ld :%s", mask, who, 
		time(NULL)+86400*2, /* Avoids filling the zline list of servers too much */
		when, reason);
#endif
}

/*************************************************************************/

void s_unsgline(char const *mask)
{
#ifdef IRC_BAHAMUT
	send_cmd(ServerName, "UNSGLINE 0 :%s", mask);
#else
	send_cmd(ServerName, "SVSNLINE - :%s", mask);
#endif	
}

/*************************************************************************/

void s_unsqline(char *mask)
{
#ifdef IRC_BAHAMUT
	send_cmd(NULL, "UNSQLINE 0 %s", mask);
#else
	send_cmd(NULL, "UNSQLINE %s", mask);
#endif	
}

/*************************************************************************/

void s_unszline(char *mask)
{
#if defined(IRC_BAHAMUT)
	s_rakill("*", mask);
#else
	send_cmd(NULL, "TKL - Z * %s %s", mask, s_OperServ);
#endif	
}

/*************************************************************************/

